"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getFigmaAssets = void 0;

var _fs = _interopRequireDefault(require("fs"));

var Figma = _interopRequireWildcard(require("figma-js"));

var _listr = _interopRequireDefault(require("listr"));

var _saveImageToFs = require("./helpers/saveImageToFs");

var _mkDirByPathSync = require("./helpers/mkDirByPathSync");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

require('@babel/polyfill');

require('dotenv').config();

var CONFIG_FILENAME = 'figma-assets-generator.json';

var getConfigFile = function getConfigFile() {
  return regeneratorRuntime.async(function getConfigFile$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.prev = 0;
          _context.next = 3;
          return regeneratorRuntime.awrap(JSON.parse(_fs["default"].readFileSync(CONFIG_FILENAME, 'utf-8')));

        case 3:
          return _context.abrupt("return", _context.sent);

        case 6:
          _context.prev = 6;
          _context.t0 = _context["catch"](0);
          return _context.abrupt("return", {});

        case 9:
        case "end":
          return _context.stop();
      }
    }
  }, null, null, [[0, 6]]);
};

var getFigmaAssets = function getFigmaAssets(options) {
  var configFile, _ref, fileId, documentId, fileExtension, personalAccessToken, output, scale, client, itemDocument, items, itemsWithUrls, getFileInfo, getItemsFromFrames, getImageURLs, saveImages, taskItems, tasks;

  return regeneratorRuntime.async(function getFigmaAssets$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          _context4.prev = 0;
          _context4.next = 3;
          return regeneratorRuntime.awrap(getConfigFile());

        case 3:
          configFile = _context4.sent;
          _ref = options || configFile, fileId = _ref.fileId, documentId = _ref.documentId, fileExtension = _ref.fileExtension, personalAccessToken = _ref.personalAccessToken, output = _ref.output, scale = _ref.scale;
          output = output || 'assets';
          fileExtension = fileExtension || 'svg';
          scale = scale || '1';
          personalAccessToken = personalAccessToken || process.env.FIGMA_TOKEN;

          if (personalAccessToken) {
            _context4.next = 11;
            break;
          }

          throw new Error('No token specified!');

        case 11:
          if (fileId) {
            _context4.next = 13;
            break;
          }

          throw new Error('No file id given');

        case 13:
          if (documentId) {
            _context4.next = 15;
            break;
          }

          throw new Error('No document id given');

        case 15:
          client = Figma.Client({
            personalAccessToken: personalAccessToken
          });
          itemDocument = {};
          items = [];
          itemsWithUrls = [];

          getFileInfo = function getFileInfo(task) {
            var file;
            return regeneratorRuntime.async(function getFileInfo$(_context2) {
              while (1) {
                switch (_context2.prev = _context2.next) {
                  case 0:
                    _context2.next = 2;
                    return regeneratorRuntime.awrap(client.file(fileId));

                  case 2:
                    file = _context2.sent;
                    itemDocument = file.data.document.children.find(function (doc) {
                      return doc.id === documentId;
                    });

                    if (itemDocument.name) {
                      _context2.next = 6;
                      break;
                    }

                    throw new Error('node id not found');

                  case 6:
                    task.title = "Found document ".concat(itemDocument.name);
                    return _context2.abrupt("return", itemDocument);

                  case 8:
                  case "end":
                    return _context2.stop();
                }
              }
            });
          };

          getItemsFromFrames = function getItemsFromFrames(task) {
            itemDocument.children.forEach(function (frame) {
              if (frame.children) {
                frame.children.filter(function (item) {
                  return item.type === 'COMPONENT';
                }).forEach(function (item) {
                  items.push({
                    id: item.id,
                    name: item.name
                  });
                });
              }
            });
            if (items.length === 0) throw new Error('No items found');
            task.title = "Found ".concat(items.length, " items");
            return items.length;
          };

          getImageURLs = function getImageURLs(task) {
            var itemIds, response, images;
            return regeneratorRuntime.async(function getImageURLs$(_context3) {
              while (1) {
                switch (_context3.prev = _context3.next) {
                  case 0:
                    itemIds = items.map(function (item) {
                      return item.id;
                    });
                    _context3.next = 3;
                    return regeneratorRuntime.awrap(client.fileImages(fileId, {
                      ids: itemIds,
                      format: fileExtension,
                      scale: scale
                    }));

                  case 3:
                    response = _context3.sent;

                    if (!response.data.err) {
                      _context3.next = 6;
                      break;
                    }

                    throw response.data.err;

                  case 6:
                    images = response.data.images;
                    itemsWithUrls = items.map(function (item) {
                      if (images.hasOwnProperty(item.id)) {
                        item.url = images[item.id];
                        return item;
                      }
                    });
                    task.title = "Found ".concat(itemsWithUrls.length, " matching URLs");
                    return _context3.abrupt("return", itemsWithUrls);

                  case 10:
                  case "end":
                    return _context3.stop();
                }
              }
            });
          };

          saveImages = function saveImages(task) {
            try {
              (0, _mkDirByPathSync.mkDirByPathSync)(output);
              itemsWithUrls.forEach(function (item, idx) {
                var name = item.name.replace(/\/|\./g, '_');
                (0, _saveImageToFs.saveImageToFs)(item.url, "".concat(name, ".").concat(fileExtension), output);
              });
              task.title = "Sucess! Saved images to '".concat(output, "'");
              return itemsWithUrls;
            } catch (e) {
              throw new Error('Error saving images to filesystem');
            }
          };

          taskItems = [{
            title: 'Getting document information',
            task: function task(ctx, _task) {
              return getFileInfo(_task);
            }
          }, {
            title: 'Find items in document',
            task: function task(ctx, _task2) {
              return getItemsFromFrames(_task2);
            }
          }, {
            title: 'Get image URLs from Figma',
            task: function task(ctx, _task3) {
              return getImageURLs(_task3);
            }
          }, {
            title: 'Save images to filesystem',
            task: function task(ctx, _task4) {
              return saveImages(_task4);
            }
          }];
          tasks = new _listr["default"](taskItems);
          _context4.next = 27;
          return regeneratorRuntime.awrap(tasks.run());

        case 27:
          _context4.next = 32;
          break;

        case 29:
          _context4.prev = 29;
          _context4.t0 = _context4["catch"](0);
          console.error(_context4.t0);

        case 32:
        case "end":
          return _context4.stop();
      }
    }
  }, null, null, [[0, 29]]);
};

exports.getFigmaAssets = getFigmaAssets;